function [variables, id, lb, ub] = out(DynareModel, constraints, variables)

    if ~iscell(constraints)
        error('Second argument must be a one-dimensional cell array.')
    end

    if iskeyvalue(constraints)
        % Calibrated variables.
        id = zeros(length(constraints)/2,1);
        j = 1;
        for i=1:2:length(constraints)
            if isempty(regexp(constraints{i}, 'G\(\w+\)'))
                tmp = find(strcmp(constraints{i}, DynareModel.endo_names));
            else
                tmp = DynareModel.endo_nbr+find(strcmp(constraints{i}(3:end-1), DynareModel.endo_names));
            end
            assert(~isempty(tmp), 'Unknown variable (%s).', constraints{i})
            id(j) = tmp;
            j = j+1;
        end
        variables(id) = [];
        lb = []; ub = [];
        return
    end

    if iskeyvalues(constraints)
        % Boundaries on variables and calibrated variables
        halfline.right = @(x, a) log(x-a);
        halfline.x = @(left, a) log(a-x);
        interval = @(x, a, b) log(x-a)-log(b-x);
        id = zeros(length(constraints)/3,1);
        lb = [-Inf(DynareModel.endo_nbr, 1); zeros(DynareModel.endo_nbr, 1)];
        ub = Inf(2*DynareModel.endo_nbr, 1);
        j = 1;
        for i=1:3:length(constraints)
            if isempty(regexp(constraints{i}, 'G\(\w+\)'))
                tmp = find(strcmp(constraints{i}, DynareModel.endo_names));
            else
                tmp = find(strcmp(constraints{i}(3:end-1), DynareModel.endo_names));
                if isempty(tmp)
                    tmp = tmp + DynareModel.endo_nbr;
                end
            end
            assert(~isempty(tmp), 'Unknown variable (%s).', constraints{i})
            lb(tmp) = constraints{i+1};
            ub(tmp) = constraints{i+2};
            if abs(lb(tmp)-ub(tmp))<1e-6
                id(j) = tmp;
            end
            j = j+1;
        end
        id = id(id>0);
        id
        for i=1:2*DynareModel.endo_nbr
            if ~ismember(i, id)
                if i>DynareModel.endo_nbr
                    assert(lb(i)<variables(i) && ub(i)>variables(i), 'Initial condition not consistent with declared boundaries (G(%s)).', DynareModel.endo_names{i-DynareModel.endo_nbr})
                else
                    assert(lb(i)<variables(i) && ub(i)>variables(i), 'Initial condition not consistent with declared boundaries (%s)', DynareModel.endo_names{i})
                end
                if isfinite(lb(i)) && isinf(ub(i))
                    variables(i) = halfline.right(variables(i), lb(i));
                elseif isinf(lb(i)) && isfinite(ub(i))
                    variables(i) = halfline.left(variables(i), ub(i));
                elseif isfinite(lb(i)) && isfinite(ub(i)) && lb(i)<ub(i)
                    variables(i) = interval(variables(i), lb(i), ub(i));
                end
            end
        end
        length(variables)
        variables(id) = [];
        length(variables)
        return
    end

    error('Second argument must be a cell array of key/value pairs or key/values triplets.')
    
end